﻿using System;
using Android.App;
using Android.Content;
using Android.Runtime;
using Android.Views;
using Android.Widget;
using Android.OS;
using System.Collections.Generic;
using System.Reflection;
using Com.GrapeCity.Xuni.FlexGrid;

namespace FlexGridAdaptive_Android
{
    [Activity(Label = "FlexGridAdaptive_Android", MainLauncher = true, Icon = "@drawable/icon")]
    public class MainActivity : Activity
    {
        FlexGrid _flexGrid;
        FlexGridDetailProvider _detailsProvider;
        GridColumn[] colArray;
        int _fitColumns;
        
        protected override void OnCreate(Bundle bundle)
        {
            base.OnCreate(bundle);

            // license
            Com.GrapeCity.Xuni.Core.LicenseManager.Key = License.Key;

            // Set our view from the "main" layout resource
            SetContentView(Resource.Layout.Main);
            
            this._flexGrid = FindViewById<FlexGrid>(Resource.Id.flexgrid);
            this._flexGrid.AutoGenerateColumns = false;
            this._flexGrid.ItemsSource = Customer.GetCustomerList(100);

            // initialize new columns
            colArray = new GridColumn[7];

            GridColumn firstNameColumn = new GridColumn(this._flexGrid, "First Name", "First");
            firstNameColumn.Width = "*";
            colArray[0] = firstNameColumn;

            GridColumn lastNameColumn = new GridColumn(this._flexGrid, "Last Name", "Last");
            lastNameColumn.Width = "*";
            colArray[1] = lastNameColumn;

            GridColumn emailColumn = new GridColumn(this._flexGrid, "Email", "Email");
            emailColumn.Width = "*";
            colArray[2] = emailColumn;

            GridColumn addressColumn = new GridColumn(this._flexGrid, "Address", "Address");
            addressColumn.Width = "*";
            colArray[3] = addressColumn;

            GridColumn cityColumn = new GridColumn(this._flexGrid, "City", "City");
            cityColumn.Width = "*";
            colArray[4] = cityColumn;

            GridColumn countryColumn = new GridColumn(this._flexGrid, "Country", "Country");
            countryColumn.Width = "*";
            colArray[5] = countryColumn;

            GridColumn postalCodeColumn = new GridColumn(this._flexGrid, "Postal Code", "PostalCode");
            postalCodeColumn.Width = "*";
            colArray[6] = postalCodeColumn;

            // determine how many columns will nicely fit the device width
            var metrics = Resources.DisplayMetrics;
            var widthInDp = ConvertPixelsToDp(metrics.WidthPixels);
            _fitColumns = widthInDp / 100;

            if(_fitColumns >= colArray.Length)
            {
                // all columns fit, no row details necessary
                _fitColumns = colArray.Length;
            }
            else
            {
                // initialize the row details provider
                _detailsProvider = new FlexGridDetailProvider(_flexGrid);
                _detailsProvider.DetailVisibilityMode = DetailVisibilityMode.ExpandMulti;
                _detailsProvider.DetailCellCreated += _detailsProvider_DetailCellCreated;
            }

            // finally, add columns that fit
            for(int i = 0; i < _fitColumns; i++)
            {
                this._flexGrid.Columns.Add(colArray[i]);
            }
        }

        /// <summary>
        /// DetailCellCreated fires when the user expands a row detail cell
        /// </summary>
        private void _detailsProvider_DetailCellCreated(object sender, DetailCellCreatedEventArgs e)
        {
            View detailView = e.Content;
            if(detailView == null)
            {
                Customer customer = (Customer)(e.Row as GridRow).DataItem;
                detailView = new FrameLayout(this.BaseContext);
                detailView.SetPadding(10, 10, 10, 10);
                LinearLayout layout = new LinearLayout(this.BaseContext);
                layout.Orientation = Orientation.Vertical;
                // calculate number of overflow columns and add them to the detail row view
                var overflowColumns = colArray.Length - _fitColumns;
                for(int i = colArray.Length - overflowColumns; i < colArray.Length; i++)
                {
                    TextView txt = new TextView(_flexGrid.Context);
                    // reflection is used to dynamically get the Customer member that the column is bound to
                    txt.Text = colArray[i].Name + ": " + customer.GetType().InvokeMember(colArray[i].Binding, BindingFlags.GetProperty, Type.DefaultBinder, customer, null).ToString();
                    txt.TextSize = 17.0f;
                    layout.AddView(txt);
                }
                ((FrameLayout)detailView).AddView(layout);
            }
            e.Content = detailView;
        }

        private int ConvertPixelsToDp(float pixelValue)
        {
            var dp = (int)((pixelValue) / Resources.DisplayMetrics.Density);
            return dp;
        }
    }
}

